// SMT-160 Sensor Demo for DIL/NetPCs (File Output Version)
// Vers. 1.00 (comdemo.c) by M.Hasewinkel / SSV Embedded Systems, Hannover
// Vers. 2.00 (28.Apr.2005) by Linxia Ye  / SSV Embedded Systems, Hannover
// Vers. 2.01 (04.May.2005) by K.D.Walter / SSV Embedded Systems, Hannover
// License GPL

// Includes

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <termios.h>                      // termios
#include <unistd.h>
#include <stdlib.h>                       // atol
#include <time.h>                         // nanosleep
#include <ctype.h>                        // toupper


// Defines

#define VERSION "2.01"                    // Program version info
#define RX_TIMEOUT      1000              // serial_rx: Wait 1s for receive
#define RX_POLLING_TIME 10                // serial_rx: Polling time (ms) from RX-Queue
#define MS_CONST        1000000L          // serial_rx: From Nanoseconds to Milliseconds


// Function: serial_open - This function is for device open and the
// serial port communication to configure

int serial_open (const char *device)
{
    struct termios termios;
    int fd;                          // Handle for serial port

    // Open serial port, no wait/no block for reading chars

    fd = open (device, O_RDWR | O_NONBLOCK);
    if (fd == -1) {
        printf ("\n  Can't read device configuration. Device unknown?\n\n");
        // perror (device);
        exit (-1);
    }

    // Get old attribute. Then setup speed, line and bit parameters

    if (tcgetattr (fd, &termios)) {
        //perror ("tcgetattr()");
        printf ("\n  Can't read device configuration. Is this a serial device?\n\n");
        exit (-1);
    }

    // Set line speed into termios structure with B9600

    if (cfsetospeed (&termios, B9600)) {
        //perror ("cfsetospeed(B9600)");
        printf ("\n  Error: cfsetospped ().\n\n");
        exit (-1);
    }

    // Disable Echo, make Data=8, Stop=1, no parity, disable all controls
    // see "man cfmakeraw". Then set new attribute

    cfmakeraw (&termios);
    if (tcsetattr (fd, TCSANOW, &termios)) {
        //perror ("tcsetattr()");
        printf ("\n  Error: tcsetattr ().\n\n");
        exit (-1);
    }
    return(fd);
}

// Function: serial_rx - This function gets n char's from the serial port

int serial_rx (int fd, unsigned char *buffer, int size)
{
    int  rd, tm ;
    struct timespec requested;

    tcflush (fd, TCIFLUSH);
    tm = RX_TIMEOUT;
    while (size > 0) {

       // Read some chars

        if ((rd = read (fd, buffer, size)) > 0) {

            // We have read somthing: Calculate pointers

            buffer += rd;
            size   -= rd;
        } else {

            // Calculate simple timeout for all characters

            if (tm >= RX_POLLING_TIME)
                tm -= RX_POLLING_TIME;
            else
                tm = 0;

            // Check for timeout

            if (!tm) {
                printf ("  RxTimeout ...\n");
                return (-1);
            }

            // Give some CPU time

            requested.tv_sec  = 0;                          // Seconds
            requested.tv_nsec = RX_POLLING_TIME * MS_CONST; // Nanoseconds
            nanosleep (&requested, NULL);
        }
    }
    return (0);
}

// Function: help - This function prints a help and usages message

int help (const char *prg_name)
{
    printf ("\n  Usage: %s [device]\n\n"
            "  %s /dev/ttyS1\n"
            "  %s /dev/ttySA1\n\n", prg_name, prg_name, prg_name);
    exit (-1);
}

// Function: main ...

int main (int argc, char *argv[])
{
    int dev;
    unsigned char rx[32];
    char device_name[32];
    FILE *fd;

    // Print version info. Check arguments

    printf ("\n  SMT-160 sensor demo - Ver. "VERSION"");
    printf ("\n  -------------------------------");
    if (argc < 2)
        help(argv[0]);

    // Build device name for serial port usage ...

    strncpy (device_name, argv[1], 31);
    device_name[31] = 0;

    // Open serial port

    printf("\n  Open device \"%s\".", device_name);
    dev = serial_open (device_name);

    // Receive data from SMT-160 sensor. Print this data out ...

    printf ("\n  Write data to /var/smt160-2.dat ...\n");
    fflush (stdout);
    serial_rx (dev, rx, 7);                         // Dummy read ...
    while (1) {
        if (serial_rx (dev, rx, 7) != -1) {
            rx[4]= 0x00;
            fd= fopen ("/var/smt160-2.dat", "w");
            if (isdigit (rx[0]) == 1)               // Check for errors ...
                fprintf (fd, "%s\n", rx);
            else
                fprintf (fd, "??.?\n");
            fflush (fd);
            fclose (fd);
        }
    }
}

// End of program
